import {MessageType} from '@/pages/proxy'
import Browser from 'webextension-polyfill'
import {ParallelTranslate} from './ParallelTranslate'

function insertProxy() {
  const container = document.createElement('div')
  const shadow = container.attachShadow({mode: 'open'})
  const iframeEl = document.createElement('iframe')
  container.id = 'botProxyContainer'
  iframeEl.id = 'botProxy'
  iframeEl.setAttribute('sandbox', 'allow-scripts allow-popups allow-same-origin')
  iframeEl.style.display = 'none'
  iframeEl.src = Browser.runtime.getURL('index.html?page=proxy')
  shadow.appendChild(iframeEl)
  document.body.appendChild(container)
  return iframeEl
}
export default class Translate {
  private $proxyIframe: HTMLIFrameElement | null = null;
  private handleMessage = (e: MessageEvent) => { // 直接初始化
    if (e.data.event === MessageType.proxyReady && this.parallelTranslate) {
      this.parallelTranslate.translate(this.$proxyIframe!, this.parallelTranslate.from, this.parallelTranslate.to, this.parallelTranslate.uid, this.parallelTranslate.app_id);
    }
  };
  private parallelTranslate: ParallelTranslate | null = null;

  constructor() {
    this.getTranslate = this.getTranslate.bind(this);
    this.removeTranslate = this.removeTranslate.bind(this);
  }

  public getTranslate(from = 'en', to = 'zh-CN', app_id = '1', uid = '1') {
    this.$proxyIframe = insertProxy()
    this.parallelTranslate = new ParallelTranslate(this.$proxyIframe!, from, to, app_id, uid);
    window.addEventListener('message', this.handleMessage);
  }
  public removeTranslate() {
    if (this.parallelTranslate) {
      this.parallelTranslate.removeIntersectionObserver();
    }
  
    window.removeEventListener('message', this.handleMessage);
  
    if (this.$proxyIframe && this.$proxyIframe instanceof HTMLIFrameElement) {
      this.$proxyIframe.remove();
    }
  
    const $container = document.getElementById('botProxyContainer') as HTMLDivElement;
    if ($container) {
      $container.remove();
    }
  }
}